class URL(object):
    SANDBOX_BASE_URL = 'https://devtopup.pgecom.com/api/'
    PRODUCTION_BASE_URL = 'https://pgecom.com/api/'
    RESOLVE = 'https://help.pgecom.com/en/'
    MONCASH = "moncash/token"
    TOKEN = "token"
    ORDER = "order/"
    
